#include once "fbgfx.bi"
#include once "gfx.bi"
#include once "chipmunk/chipmunk.bi"


declare function main( byval argc as integer, byval arv as zstring ptr ptr ) as integer


end main( __fb_argc__, __fb_argv__ )


function main( byval argc as integer, byval arv as zstring ptr ptr ) as integer
	
	screenres 640, 480, 32,,FB.GFX_HIGH_PRIORITY
	
	'initialize the Chipmunk space and give it some global gravity
	dim as cpSpace ptr cpWorld = cpSpaceNew()
	cpSpaceSetGravity( cpWorld, cpv(0, 9.8) )
	
	'setup a rigid body container
	'since we know this container will contain a cirlce shaped object, 
	'we'll use Chipmunk's built-in moment of inertia calculator
	dim as cpFloat moi = cpMomentForCircle( 25, 0, 64, cpv(20,0) )
	dim as cpBody ptr rigidBody = cpBodyNew( 25, moi )
	
	'now we'll actually add the rigid body to the world, even though it has nothing attached yet
	cpSpaceAddBody( cpWorld, rigidBody)

	'now we'll actually create a shape and attach it to the rigidBody continer we just created
	dim as cpShape ptr ballShape = cpCircleShapeNew(rigidBody, 64, cpV(0,0) )
	
	'tell Chipmunk how "bouncy" we want the ball to be 
	cpShapeSetElasticity(ballShape, .75)
	
	'tell Chipmunk how "rough" we want the ball to be  
	cpShapeSetFriction(ballShape, 1)
	
	'finally, we can add the shape to the world, 
	cpSpaceAddShape( cpWorld, ballShape)
	
	'and set a starting position
	cpBodySetPosition( rigidBody, cpv(320, 250) )
	
	
	'next, we'll setup some verrtices to use for the walls and ground
	dim as cpVect boxVerts(4)
	boxVerts(0) = cpv(0,0)
	boxVerts(1) = cpv(0,420)
	boxVerts(2) = cpv(320,479)
	boxVerts(3) = cpv(639,420)
	boxVerts(4) = cpv(639,0)
	
	'since this shape will only be used for static collisions, we don't need to create a rigid body
	'instead, we'll just grab a reference to the built-in static body and attach to that 
	dim as cpShape ptr lineShape(ubound(boxVerts))
	
	for l as integer = 0 to ubound(boxverts)
		
		dim as integer lAnd1 = (l+1) mod (ubound(boxVerts)+1) 
		
		lineShape(l) = cpSegmentShapeNew( cpSpaceGetStaticBody(cpWorld), boxVerts(l), boxVerts(lAnd1), 1 )
		cpShapeSetElasticity( lineShape(l), .95 )
		cpShapeSetFriction( lineShape(l), .25 )
		cpSpaceAddShape( cpWorld, lineShape(l) )
		
	next
	
	dim as FB.IMAGE ptr testi = imagecreate( 128, 128 )
	bload "res/gfx/test_sprite.bmp", testi

	'just some variables for time keeping, etc...
	dim as double frameStep = 180, simRate = 20
	dim as double frequency = 1.0/frameStep
	dim as double curTime, accumStep, deltaTime
	dim as double jumpVal
	
	
	
	curTime = timer
	do
		
		'if spacebar is pressed, set jumpVal to 
		if multikey(FB.SC_SPACE) then
			
			jumpVal = -cpBodyGetMass(rigidBody)*50
			
		end if

		'get the frame timing and run the simulation loop
		deltaTime = timer - curTime
		accumStep += deltaTime
		
		do
			
			if jumpVal <> 0 then
				
				cpBodyApplyImpulseAtWorldPoint( rigidBody, cpv(0,jumpVal), cpBodyGetPosition(rigidBody) )
				
				jumpVal = 0
				
			end if
			
			cpSpaceStep( cpWorld, frequency*simRate )
	
			accumStep -= frequency

		loop while accumStep >= frequency

		curTime = timer
		
		'render the gfx...
		screenlock
		cls

		for l as integer = 0 to ubound(boxVerts)
			
			dim as integer lAnd1 = (l+1) mod (ubound(boxVerts)+1)
			
			dim as cpVect p1 = boxVerts(l)
			dim as cpVect p2 = boxVerts(lAnd1)

			line( p1.x, p1.y )-( p2.x, p2.y), rgb(255,255,0)

		next
		
		'get the position and rotation from the rigid body the ball shape is attached to,
		'and pass the values to rotozoom for rendering
		dim as cpVect position = cpBodyGetPosition(rigidBody)
		dim as cpFloat angle = cpBodyGetAngle(rigidBody)
		rotozoom( 0, testi, position.x, position.y, -angle, 1, 1 )
		
		dim as string tellMe = "PRESS SPACE TO " + chr(34) + "JUMP!" + chr(34)
		draw string (320-len(tellMe)*4, 240), tellMe,rgb(255,255,0),,XOR 
		
		screensync
		screenunlock

		sleep 3, 1

	loop until multikey(FB.SC_ESCAPE)

	'free up the all the memory we used
	imagedestroy(testi)
	
	cpShapeFree(ballShape)
	
	for l as integer = 0 to ubound(lineShape)
		
		cpShapeFree( lineShape(l) )
		
	next
	
	cpBodyFree(rigidBody)
	
	cpSpaceFree(cpWorld)

	return argc

end function
